@@ -6,7 +6,6 @@ from argparse import ArgumentParser, HelpFormatter
 from logging import CRITICAL, DEBUG, ERROR, INFO, WARNING
 from os import listdir
 from os.path import exists, isfile, isdir
-from tempfile import gettempdir
 
 from .adapters import names as adapter_names
 from .target import available as available_targets
@@ -83,14 +82,12 @@ class CommonArgs(object):
             help="Number of iterations performed before relaunching the browser (default: %(default)s)")
         self.launcher_grp.add_argument(
             "-t", "--timeout", type=int, default=60,
-            help="Iteration timeout in seconds (default: %(default)s)")
+            help="Iteration or test case timeout in seconds (default: %(default)s)."
+                 " Browser build types and debuggers can affect the amount of time"
+                 " required to run a test case.")
         self.launcher_grp.add_argument(
             "--valgrind", action="store_true",
             help="Use Valgrind (Linux only)")
-        self.launcher_grp.add_argument(
-            "-w", "--working-path",
-            help="Working directory. Intended to be used with ram-drives."
-                 " (default: %r)" % gettempdir())
         self.launcher_grp.add_argument(
             "--xvfb", action="store_true",
             help="Use Xvfb (Linux only)")
@@ -107,6 +104,9 @@ class CommonArgs(object):
             "--tool",
             help="Override tool name used when reporting issues to FuzzManager")
 
+        self.parser.epilog = "For addition help check out the wiki:" \
+            " https://github.com/MozillaSecurity/grizzly/wiki"
+
     def parse_args(self, argv=None):
         args = self.parser.parse_args(argv)
         self.sanity_check(args)
@@ -148,9 +148,6 @@ class CommonArgs(object):
         if args.relaunch < 1:
             self.parser.error("--relaunch must be >= 1")
 
-        if args.working_path is not None and not isdir(args.working_path):
-            self.parser.error("%r is not a directory" % args.working_path)
-
         if args.extension:
             for ext in args.extension:
                 if not exists(ext):
@@ -201,9 +198,6 @@ class GrizzlyArgs(CommonArgs):
             "--s3-fuzzmanager", action="store_true",
             help="Report large attachments (if any) to S3 and then the crash & S3 link to FuzzManager")
 
-        self.parser.epilog = "For addition help check out the wiki:" \
-            " https://github.com/MozillaSecurity/grizzly/wiki"
-
     def sanity_check(self, args):
         super(GrizzlyArgs, self).sanity_check(args)
 
@@ -220,3 +214,6 @@ class GrizzlyArgs(CommonArgs):
 
         if args.tool is not None and not (args.fuzzmanager or args.s3_fuzzmanager):
             self.parser.error("--tool can only be given with --fuzzmanager/--s3-fuzzmanager")
+
+        if args.rr and args.valgrind:
+            self.parser.error("'--rr' and '--valgrind' cannot be used together")
