@@ -6,14 +6,15 @@
 from .adapter import Adapter, AdapterError
 from .iomanager import IOManager, ServerMap
 from .reporter import FilesystemReporter, FuzzManagerReporter, Report, Reporter, S3FuzzManagerReporter
-from .runner import Runner
+from .runner import Runner, RunResult
 from .status import ReducerStats, Status
 from .storage import TestCaseLoadFailure, TestCase, TestFile, TestFileExists
+from .utils import grz_tmp
 
 
 __all__ = (
-    "Adapter", "AdapterError", "FilesystemReporter", "FuzzManagerReporter", "IOManager",
-    "ReducerStats", "Report", "Reporter", "Runner", "S3FuzzManagerReporter", "ServerMap", "Status",
-    "TestCase", "TestCaseLoadFailure", "TestFile", "TestFileExists")
+    "Adapter", "AdapterError", "FilesystemReporter", "FuzzManagerReporter", "grz_tmp", "IOManager",
+    "ReducerStats", "Report", "Reporter", "Runner", "RunResult", "S3FuzzManagerReporter",
+    "ServerMap", "Status", "TestCase", "TestCaseLoadFailure", "TestFile", "TestFileExists")
 __author__ = "Jesse Schwartzentruber"
 __credits__ = ["Jesse Schwartzentruber", "Tyson Smith"]
