@@ -26,12 +26,11 @@ class IOManager(object):
         "MOZ_CHAOSMODE",
         "XPCOM_DEBUG_BREAK")
 
-    def __init__(self, report_size=1, working_path=None):
+    def __init__(self, report_size=1):
         assert report_size > 0
         self.harness = None
         self.server_map = ServerMap()  # manage redirects, include directories and dynamic responses
         self.tests = deque()
-        self.working_path = working_path
         self._environ_files = list()  # collection of files that should be added to the testcase
         self._generated = 0  # number of test cases generated
         self._report_size = report_size
