@@ -3,16 +3,19 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 from logging import getLogger
+from shutil import rmtree
+from tempfile import mkdtemp
 from time import sleep, time
 
 from sapphire import SERVED_TIMEOUT
 from ..target import TargetLaunchTimeout
+from .utils import grz_tmp
 
-__all__ = ("Runner",)
+__all__ = ("Runner", "RunResult")
 __author__ = "Tyson Smith"
 __credits__ = ["Tyson Smith"]
 
-LOG = getLogger("grz_runner")
+LOG = getLogger(__name__)
 
 # _IdleChecker is used to help determine if the target is hung (actively using CPU)
 # or if it has not made expected the HTTP requests for other reasons (idle).
@@ -27,7 +30,7 @@ class _IdleChecker(object):
         assert callable(check_cb)
         assert initial_delay >= 0
         assert poll_delay >= 0
-        assert threshold >= 0
+        assert 100 > threshold >= 0
         self._check_cb = check_cb  # callback used to check if target is idle
         self._init_delay = initial_delay  # time to wait before the initial idle poll
         self._poll_delay = poll_delay  # time to wait between subsequent polls
@@ -71,23 +74,17 @@ class _IdleChecker(object):
 
 
 class Runner(object):
-    COMPLETE = 1
-    ERROR = 2
-    FAILED = 3
-    IGNORED = 4
+    __slots__ = ("_idle", "_server", "_target")
 
-    __slots__ = ("_idle", "_server", "_target", "result", "served", "timeout")
-
-    def __init__(self, server, target, idle_threshold=0, idle_delay=60):
+    def __init__(self, server, target, idle_threshold=0, idle_delay=0):
         if idle_threshold > 0:
+            assert idle_delay > 0
+            LOG.debug("using idle check, th %d, delay %ds", idle_threshold, idle_delay)
             self._idle = _IdleChecker(target.is_idle, idle_threshold, idle_delay)
         else:
             self._idle = None
         self._server = server  # a sapphire instance to serve the test case
         self._target = target  # target to run test case
-        self.result = None
-        self.served = None
-        self.timeout = False
 
     def launch(self, location, env_mod=None, max_retries=3, retry_delay=0):
         """Launch a target and open `location`.
@@ -148,68 +145,96 @@ class Runner(object):
             return "?".join([location, "&".join(args)])
         return location
 
-    def run(self, ignore, server_map, testcase, coverage=False, wait_for_callback=False):
+    def run(self, ignore, server_map, testcase, test_path=None, coverage=False, wait_for_callback=False):
         """Serve a testcase and monitor the target for results.
 
         Args:
             ignore (list): List of failure types to ignore.
             server_map (sapphire.ServerMap): A ServerMap.
             testcase (grizzly.TestCase): The test case that will be served.
+            test_path (str): Location of test case data on the filesystem.
             coverage (bool): Trigger coverage dump.
             wait_for_callback: (bool): Use `_keep_waiting()` to indicate when
                                        framework should move on.
 
         Returns:
-            None
+            RunResult: Files served, status and timeout flag from the run.
         """
-        # set initial state
-        self.served = None
-        self.result = None
-        self.timeout = False
         if self._idle is not None:
             self._idle.schedule_poll(initial=True)
-        # serve the test case
-        server_status, self.served = self._server.serve_testcase(
-            testcase,
-            continue_cb=self._keep_waiting,
-            forever=wait_for_callback,
-            server_map=server_map)
+        try:
+            # unpack test case
+            if test_path is None:
+                wwwdir = mkdtemp(prefix="test_", dir=grz_tmp("serve"))
+                testcase.dump(wwwdir)
+            else:
+                wwwdir = test_path
+            # serve the test case
+            serve_start = time()
+            server_status, served = self._server.serve_path(
+                wwwdir,
+                continue_cb=self._keep_waiting,
+                forever=wait_for_callback,
+                optional_files=tuple(testcase.optional),
+                server_map=server_map)
+            duration = time() - serve_start
+        finally:
+            # remove temporary files
+            if test_path is None:
+                rmtree(wwwdir)
+        result = RunResult(served, duration, timeout=server_status == SERVED_TIMEOUT)
+        # TODO: fix calling TestCase.add_batch() for multi-test replay
         # add all include files that were served
         for url, resource in server_map.include.items():
-            testcase.add_batch(resource.target, self.served, prefix=url)
-        self.timeout = server_status == SERVED_TIMEOUT
-        served_lpage = testcase.landing_page in self.served
+            testcase.add_batch(resource.target, result.served, prefix=url)
+        served_lpage = testcase.landing_page in result.served
         if not served_lpage:
             LOG.debug("%r not served!", testcase.landing_page)
-        elif coverage and not self.timeout:
+        elif coverage and not result.timeout:
             # dump_coverage() should be called before detect_failure()
             # to help catch any coverage related issues.
             self._target.dump_coverage()
         # detect failure
-        failure_detected = self._target.detect_failure(ignore, self.timeout)
+        failure_detected = self._target.detect_failure(ignore, result.timeout)
         if failure_detected == self._target.RESULT_FAILURE:
-            self.result = self.FAILED
+            result.status = RunResult.FAILED
         elif not served_lpage:
             # something is wrong so close the target
             # previous iteration put target in a bad state?
             self._target.close()
-            self.result = self.ERROR
+            result.status = RunResult.ERROR
         elif failure_detected == self._target.RESULT_IGNORED:
-            self.result = self.IGNORED
+            result.status = RunResult.IGNORED
         else:
-            self.result = self.COMPLETE
+            result.status = RunResult.COMPLETE
+        return result
 
     def _keep_waiting(self):
-        """Callback used by the server to determine if should continue to wait
-        for the requests from the target.
+        """Callback used by the server to determine if it should continue to
+        wait for the requests from the target.
 
         Args:
             None
 
         Returns:
-            bool: Continue to serve test test case
+            bool: Continue to serve the test case.
         """
         if self._idle is not None and self._idle.is_idle():
             LOG.debug("idle target detected")
             return False
         return self._target.monitor.is_healthy()
+
+
+class RunResult(object):
+    COMPLETE = 1
+    ERROR = 2
+    FAILED = 3
+    IGNORED = 4
+
+    __slots__ = ("duration", "served", "status", "timeout")
+
+    def __init__(self, served, duration, status=None, timeout=False):
+        self.duration = duration
+        self.served = served
+        self.status = status
+        self.timeout = timeout
