@@ -8,9 +8,14 @@ from itertools import chain
 import json
 import os
 import shutil
-import tempfile
+from tempfile import mkdtemp, SpooledTemporaryFile
+from time import time
+from zipfile import BadZipfile, ZipFile
+from zlib import error as zlib_error
 
 from ..target import sanitizer_opts
+from .utils import grz_tmp
+
 
 __all__ = ("TestCase", "TestFile", "TestCaseLoadFailure", "TestFileExists")
 __author__ = "Tyson Smith"
@@ -31,15 +36,16 @@ TestFileMap = namedtuple("TestFileMap", "meta optional required")
 class TestCase(object):
     __slots__ = (
         "adapter_name", "duration", "env_vars", "input_fname", "landing_page",
-        "redirect_page", "_existing_paths", "_files")
+        "redirect_page", "timestamp", "_existing_paths", "_files")
 
-    def __init__(self, landing_page, redirect_page, adapter_name, input_fname=None):
+    def __init__(self, landing_page, redirect_page, adapter_name, input_fname=None, timestamp=None):
         self.adapter_name = adapter_name
         self.duration = None
         self.env_vars = dict()  # environment variables
         self.input_fname = input_fname  # file that was used to create the test case
         self.landing_page = landing_page
         self.redirect_page = redirect_page
+        self.timestamp = time() if timestamp is None else timestamp
         self._existing_paths = list()  # file paths in use
         self._files = TestFileMap(
             meta=list(),  # environment files such as prefs.js, etc...
@@ -56,8 +62,8 @@ class TestCase(object):
         """Add a test file to test case and perform sanity checks.
 
         Args:
-            target (list): Specific list of Files to append target test_file to.
-            test_file (TestFile): TestFile to append
+            target (list): Specific list of files to append target test_file to.
+            test_file (TestFile): TestFile to append.
 
         Returns:
             None
@@ -70,14 +76,14 @@ class TestCase(object):
 
     def add_batch(self, path, include_files, prefix=None):
         """Iterate over files in include_files and attach the files that are
-        located in path to testcase.
+        located in path to TestCase.
 
         Args:
             path (str): Path to the root of the directory that contains files.
             include_files (iterable): Paths of the files to be added to the
-                                      test case if they exist in path.
+                                      TestCase if they exist in path.
             prefix (str): Path prefix to prepend to file when adding to
-                          test case.
+                          the TestCase.
 
         Returns:
             None
@@ -92,10 +98,10 @@ class TestCase(object):
             self.add_from_file(fname, file_name=test_path)
 
     def add_meta(self, meta_file):
-        """Add a test file to test case as a meta file.
+        """Add a TestFile to TestCase as a meta file.
 
         Args:
-            meta_file (TestFile): TestFile to add to TestCase
+            meta_file (TestFile): TestFile to add to TestCase.
 
         Returns:
             None
@@ -103,11 +109,11 @@ class TestCase(object):
         self._add(self._files.meta, meta_file)
 
     def add_environ_var(self, name, value):
-        """Add environment variable to test case.
+        """Add environment variable to TestCase.
 
         Args:
-            name (str): Environment variable name
-            value (str): Environment variable value
+            name (str): Environment variable name.
+            value (str): Environment variable value.
 
         Returns:
             None
@@ -115,11 +121,11 @@ class TestCase(object):
         self.env_vars[name] = value
 
     def add_file(self, test_file, required=True):
-        """Add a test file to test case.
+        """Add a TestFile to TestCase.
 
         Args:
-            meta_file (TestFile): TestFile to add to TestCase
-            required (bool): Indicates if test file must be served
+            meta_file (TestFile): TestFile to add to TestCase.
+            required (bool): Indicates if test file must be served.
 
         Returns:
             None
@@ -130,13 +136,14 @@ class TestCase(object):
             self._add(self._files.optional, test_file)
 
     def add_from_data(self, data, file_name, encoding="UTF-8", required=True):
-        """Create a TestFile and add it to the test case.
+        """Create a TestFile and add it to the TestCase.
 
         Args:
-            data (bytes): Data to write to file
-            file_name (str): Name for the test file
-            encoding (str): Encoding to be used
-            required (bool): Indicates if test file must be served
+            data (bytes or str): Data to write to file. If data is of type str
+                                 encoding must be given.
+            file_name (str): Name for the TestFile.
+            encoding (str): Encoding to be used.
+            required (bool): Indicates whether the TestFile must be served.
 
         Returns:
             None
@@ -149,12 +156,13 @@ class TestCase(object):
             raise
 
     def add_from_file(self, input_file, file_name=None, required=True):
-        """Create a TestFile from an existing file and add it to the test case.
+        """Create a TestFile from an existing file and add it to the TestCase.
 
         Args:
-            input_file (str): Path to existing file to use
-            file_name (str): Name for the test file
-            required (bool): Indicates if test file must be served
+            input_file (str): Path to existing file to use.
+            file_name (str): Name for the TestFile. If file_name is not given
+                             the name of the input_file will be used.
+            required (bool): Indicates whether the TestFile must be served.
 
         Returns:
             None
@@ -186,7 +194,7 @@ class TestCase(object):
             file_name (str): File name to search for in TestCase.
 
         Returns:
-            bool: True if file exists in the TestCase otherwise False
+            bool: True if file exists in the TestCase otherwise False.
         """
         return file_name in self._existing_paths
 
@@ -209,8 +217,8 @@ class TestCase(object):
         """Write all the test case data to the filesystem.
 
         Args:
-            out_path (str): Path to directory to output data
-            include_details (bool): Output "test_info.json" file
+            out_path (str): Path to directory to output data.
+            include_details (bool): Output "test_info.json" file.
 
         Returns:
             None
@@ -227,13 +235,75 @@ class TestCase(object):
                 "duration": self.duration,
                 "env": self.env_vars,
                 "input": os.path.basename(self.input_fname) if self.input_fname else None,
-                "target": self.landing_page}
+                "target": self.landing_page,
+                "timestamp": self.timestamp}
             with open(os.path.join(out_path, "test_info.json"), "w") as out_fp:
                 json.dump(info, out_fp, indent=2, sort_keys=True)
             # save meta files
             for meta_file in self._files.meta:
                 meta_file.dump(out_path)
 
+    def get_file(self, file_name):
+        """Look up and return the TestFile with the specified file name.
+
+        Args:
+            file_name (str): Name of file to retrieve.
+
+        Returns:
+            TestFile: TestFile with matching file name otherwise None.
+        """
+        for tfile in chain(self._files.meta, self._files.optional, self._files.required):
+            if tfile.file_name == file_name:
+                return tfile
+        return None
+
+    @classmethod
+    def load(cls, path, load_prefs, adjacent=False):
+        """Load TestCases from disk.
+
+        Args:
+            path (str): Path can be:
+                        1) A directory containing `test_info.json` and data.
+                        2) A directory with one or more subdirectories of 1.
+                        3) A zip archive containing testcase data or
+                           subdirectories containing testcase data.
+                        4) A single file to be used as a test case.
+            load_prefs (bool): Load prefs.js file if available.
+            adjacent (str): Load adjacent files as part of the test case.
+                            This is always the case when loading a directory.
+                            WARNING: This should be used with caution!
+
+        Returns:
+            list: TestCases successfully loaded from path.
+        """
+        # unpack archive if needed
+        if path.lower().endswith(".zip"):
+            unpacked = mkdtemp(prefix="unpack_", dir=grz_tmp("storage"))
+            try:
+                with ZipFile(path) as zip_fp:
+                    zip_fp.extractall(path=unpacked)
+            except (BadZipfile, zlib_error):
+                shutil.rmtree(unpacked, ignore_errors=True)
+                raise TestCaseLoadFailure("Testcase archive is corrupted") from None
+            path = unpacked
+        else:
+            unpacked = None
+        # load testcase data from disk
+        try:
+            if os.path.isfile(path):
+                tests = [cls.load_single(path, load_prefs, adjacent=adjacent)]
+            elif os.path.isdir(path):
+                tests = list()
+                for tc_path in TestCase.scan_path(path):
+                    tests.append(cls.load_single(tc_path, load_prefs))
+                tests.sort(key=lambda tc: tc.timestamp)
+            else:
+                raise TestCaseLoadFailure("Invalid TestCase path")
+        finally:
+            if unpacked is not None:
+                shutil.rmtree(unpacked, ignore_errors=True)
+        return tests
+
     def load_environ(self, path, env_data):
         # sanity check environment variable data
         for name, value in env_data.items():
@@ -250,79 +320,49 @@ class TestCase(object):
                 self.env_vars[opt_key] = ":".join("=".join((k, v)) for k, v in opts.items())
 
     @classmethod
-    def load_path(cls, path, full_scan=False, prefs=True):
+    def load_single(cls, path, load_prefs, adjacent=False):
         """Load contents of a TestCase from disk. If `path` is a directory it must
-        contain a valid test_info.json file.
+        contain a valid 'test_info.json' file.
 
         Args:
             path (str): Path to the directory or file to load.
-            full_scan (bool): Include all files in the directory containing the
-                              test case entry point as well as the contents of
-                              subdirectories. This is always the case when
-                              loading a directory.
-                              WARNING: This should be used with caution!
-            prefs (bool): Include prefs.js file in the test case.
+            load_prefs (bool): Load prefs.js file if available.
+            adjacent (str): Load adjacent files as part of the test case.
+                            This is always the case when loading a directory.
+                            WARNING: This should be used with caution!
 
         Returns:
             TestCase: A TestCase.
         """
         path = os.path.abspath(path)
         if os.path.isdir(path):
-            # load a directory using test_info.json
+            # load using test_info.json
             try:
                 with open(os.path.join(path, "test_info.json"), "r") as in_fp:
                     info = json.load(in_fp)
             except IOError:
-                raise TestCaseLoadFailure("Missing 'test_info.json'")
+                raise TestCaseLoadFailure("Missing 'test_info.json'") from None
             except ValueError:
-                raise TestCaseLoadFailure("Invalid 'test_info.json'")
-            if "target" not in info:
-                raise TestCaseLoadFailure("'test_info.json' missing 'target' entry")
+                raise TestCaseLoadFailure("Invalid 'test_info.json'") from None
+            if not isinstance(info.get("target"), str):
+                raise TestCaseLoadFailure("'test_info.json' has invalid 'target' entry")
             entry_point = os.path.basename(info["target"])
             if not os.path.isfile(os.path.join(path, entry_point)):
-                raise TestCaseLoadFailure("entry_point '%s' not found in '%s'" % (entry_point, path))
-            adapter = info.get("adapter", None)
-            full_scan = True
+                raise TestCaseLoadFailure("Entry point %r not found in '%s'" % (entry_point, path))
+            # always load all contents of a directory if a 'test_info.json' is loaded
+            adjacent = True
         elif os.path.isfile(path):
-            adapter = None
             entry_point = os.path.basename(path)
+            info = dict()
             path = os.path.dirname(path)
-            info = None
         else:
-            raise TestCaseLoadFailure("Cannot find %r" % (path,))
-        test = cls(None, None, adapter)
-        if full_scan:
-            # load all files from directory as test
-            for dpath, _, files in os.walk(path):
-                for fname in files:
-                    if fname == "test_info.json":
-                        continue
-                    if dpath == path:
-                        if fname == "prefs.js":
-                            if prefs:
-                                test.add_meta(TestFile.from_file(os.path.join(dpath, fname)))
-                            continue
-                        if fname == entry_point:
-                            test.add_from_file(os.path.join(dpath, fname))
-                            # set entry point
-                            test.landing_page = fname
-                            continue
-                        location = None
-                    else:
-                        # handle nested directories
-                        location = "/".join((dpath.split(path, 1)[-1], fname))
-                    test.add_from_file(
-                        os.path.join(dpath, fname),
-                        file_name=location,
-                        required=False)
-        else:
-            # load single file as test
-            test.add_from_file(os.path.join(path, entry_point))
-            test.landing_page = entry_point
-        if test.landing_page is None:  # pragma: no cover
-            # this should not be possible
-            test.cleanup()
-            raise AssertionError("Scanning for test case 'entry point' failed")
+            raise TestCaseLoadFailure("Missing or invalid TestCase %r" % (path,))
+        # create testcase and add data
+        test = cls(None, None, info.get("adapter", None), timestamp=info.get("timestamp", 0))
+        if load_prefs and os.path.isfile(os.path.join(path, "prefs.js")):
+            test.add_meta(TestFile.from_file(os.path.join(path, "prefs.js")))
+        test.add_from_file(os.path.join(path, entry_point))
+        test.landing_page = entry_point
         # load environment variables
         if info:
             try:
@@ -330,17 +370,29 @@ class TestCase(object):
             except TestCaseLoadFailure:
                 test.cleanup()
                 raise
+        # load all adjacent data from directory
+        if adjacent:
+            for dpath, _, files in os.walk(path):
+                for fname in files:
+                    # ignore files that have been previously loaded
+                    if fname in (entry_point, "prefs.js", "test_info.json"):
+                        continue
+                    location = "/".join((dpath.split(path, 1)[-1], fname))
+                    test.add_from_file(
+                        os.path.join(dpath, fname),
+                        file_name=location,
+                        required=False)
         return test
 
     @property
     def optional(self):
-        """Get file names of optional TestFiles
+        """Get file names of optional TestFiles.
 
         Args:
             None
 
-        Returns:
-            generator: file names (str) of optional files
+        Yields:
+            str: File names of optional files.
         """
         for test in self._files.optional:
             yield test.file_name
@@ -354,32 +406,72 @@ class TestCase(object):
         Returns:
             None
         """
-        keep = set(keep)
-        to_remove = []
-        for idx, tfile in enumerate(self._files.optional):
-            if tfile.file_name not in keep:
+        opt_files = tuple(x.file_name for x in self._files.optional)
+        if not opt_files:
+            # nothing to purge
+            return
+        # filter required files from opt_files files to keep
+        keep_opt = list()
+        for fname in set(keep):
+            if fname not in (x.file_name for x in self._files.required):
+                keep_opt.append(fname)
+        # sanity check keep (cannot remove file that does not exist)
+        assert all(fname in opt_files for fname in keep_opt)
+        # purge
+        to_remove = list()
+        for idx, fname in enumerate(opt_files):
+            if fname not in keep_opt:
                 to_remove.append(idx)
         for idx in reversed(to_remove):
             self._files.optional.pop(idx).close()
 
+    @staticmethod
+    def scan_path(path):
+        """Check path and subdirectories for potential test cases.
+
+        Args:
+            path (str): Path to scan.
+
+        Yields:
+            str: Path to what appears to be a valid testcase.
+        """
+        contents = os.listdir(path)
+        if "test_info.json" in contents:
+            yield path
+        else:
+            for entry in contents:
+                tc_path = os.path.join(path, entry)
+                if os.path.isfile(os.path.join(tc_path, "test_info.json")):
+                    yield tc_path
+
 
 class TestFile(object):
     CACHE_LIMIT = 0x80000  # data cache limit per file: 512KB
     XFER_BUF = 0x10000  # transfer buffer size: 64KB
 
-    __slots__ = ("_fp", "file_name")
+    __slots__ = ("_file_name", "_fp")
 
     def __init__(self, file_name):
-        if not file_name:
-            raise TypeError("TestFile requires a name")
-        self._fp = tempfile.SpooledTemporaryFile(max_size=self.CACHE_LIMIT, prefix="grz_tf_")
-        # TODO: Add file_name sanitation since it is used when the file is written to the fs
-        # XXX: This is a naive fix for a larger path issue
+        # This is a naive fix for a larger path issue. This is a simple sanity
+        # check and does not check if invalid characters are used. If an invalid
+        # file name is used an exception will be raised when trying to write
+        # that file to the file system.
         if "\\" in file_name:
             file_name = file_name.replace("\\", "/")
         if file_name.startswith("/"):
             file_name = file_name.lstrip("/")
-        self.file_name = os.path.normpath(file_name)  # name including path relative to wwwroot
+        if file_name.endswith("."):
+            file_name = file_name.rstrip(".")
+        if not file_name \
+                or ("/" in file_name and not file_name.rsplit("/", 1)[-1]) \
+                or file_name.startswith("../"):
+            raise TypeError("file_name is invalid %r" % (file_name,))
+        # name including path relative to wwwroot
+        self._file_name = os.path.normpath(file_name)
+        self._fp = SpooledTemporaryFile(
+            dir=grz_tmp("storage"),
+            max_size=self.CACHE_LIMIT,
+            prefix="testfile_")
 
     def __enter__(self):
         return self
@@ -396,7 +488,7 @@ class TestFile(object):
         Returns:
             TestFile: A copy of the TestFile instance
         """
-        cloned = TestFile(self.file_name)
+        cloned = TestFile(self._file_name)
         self._fp.seek(0)
         shutil.copyfileobj(self._fp, cloned._fp, self.XFER_BUF)  # pylint: disable=protected-access
         return cloned
@@ -408,7 +500,7 @@ class TestFile(object):
             None
 
         Returns:
-            None TestFile instance
+            None
         """
         self._fp.close()
 
@@ -429,32 +521,37 @@ class TestFile(object):
         return data
 
     def dump(self, path):
-        """Write test file data to the filesystem.
+        """Write TestFile data to the filesystem.
 
         Args:
-            path (str): Path to output data
+            path (str): Path to output data.
 
         Returns:
             None
         """
-        target_path = os.path.join(path, os.path.dirname(self.file_name))
+        target_path = os.path.join(path, os.path.dirname(self._file_name))
         if not os.path.isdir(target_path):
             os.makedirs(target_path)
         self._fp.seek(0)
-        with open(os.path.join(path, self.file_name), "wb") as dst_fp:
+        with open(os.path.join(path, self._file_name), "wb") as dst_fp:
             shutil.copyfileobj(self._fp, dst_fp, self.XFER_BUF)
 
+    @property
+    def file_name(self):
+        return self._file_name
+
     @classmethod
     def from_data(cls, data, file_name, encoding="UTF-8"):
         """Create a TestFile and add it to the test case.
 
         Args:
-            data (bytes): Data to write to file
-            file_name (str): Name for the test file
-            encoding (str): Encoding to be used
+            data (bytes or str): Data to write to file. If data is of type str
+                                 encoding must be given.
+            file_name (str): Name for the TestFile.
+            encoding (str): Encoding to be used.
 
         Returns:
-            TestFile: new instance
+            TestFile: A TestFile.
         """
         t_file = cls(file_name)
         if data:
@@ -469,11 +566,12 @@ class TestFile(object):
         """Create a TestFile from an existing file.
 
         Args:
-            input_file (str): Path to existing file to use
-            file_name (str): Name for the test file
+            input_file (str): Path to existing file to use.
+            file_name (str): Name for the TestFile. If file_name is not given
+                             the name of the input_file will be used.
 
         Returns:
-            TestFile: new instance
+            TestFile: A TestFile.
         """
         if file_name is None:
             file_name = os.path.basename(input_file)
@@ -502,7 +600,7 @@ class TestFile(object):
         """Add data to the TestFile.
 
         Args:
-            data (bytes): Data to add to the TestFile
+            data (bytes): Data to add to the TestFile.
 
         Returns:
             None
