@@ -3,11 +3,15 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 from logging import basicConfig, DEBUG, getLogger
+from os.path import join as pathjoin
+from os import getcwd
 
 from sapphire import Sapphire
 
 from .adapters import get as get_adapter
-from .common import FilesystemReporter, FuzzManagerReporter, IOManager, S3FuzzManagerReporter
+from .common.iomanager import IOManager
+from .common.reporter import FilesystemReporter, FuzzManagerReporter, S3FuzzManagerReporter
+from .common.utils import grz_tmp
 from .session import Session
 from .target import load as load_target, TargetLaunchError, TargetLaunchTimeout
 
@@ -16,14 +20,16 @@ __author__ = "Tyson Smith"
 __credits__ = ["Tyson Smith", "Jesse Schwartzentruber"]
 
 
-log = getLogger("grizzly")  # pylint: disable=invalid-name
+log = getLogger(__name__)  # pylint: disable=invalid-name
 
 def configure_logging(log_level):
     if log_level == DEBUG:
-        log_fmt = "%(levelname).1s %(name)s [%(asctime)s] %(message)s"
+        date_fmt = None
+        log_fmt = "%(asctime)s %(levelname).1s %(name)s | %(message)s"
     else:
+        date_fmt = "%Y-%m-%d %H:%M:%S"
         log_fmt = "[%(asctime)s] %(message)s"
-    basicConfig(format=log_fmt, datefmt="%Y-%m-%d %H:%M:%S", level=log_level)
+    basicConfig(format=log_fmt, datefmt=date_fmt, level=log_level)
 
 def main(args):
     configure_logging(args.log_level)
@@ -37,10 +43,10 @@ def main(args):
         log.info("Ignoring: %s", ", ".join(args.ignore))
     if args.xvfb:
         log.info("Running with Xvfb")
-    if args.valgrind:
-        log.info("Running with Valgrind. This will be SLOW!")
     if args.rr:
         log.info("Running with RR")
+    elif args.valgrind:
+        log.info("Running with Valgrind. This will be SLOW!")
 
     adapter = None
     iomanager = None
@@ -49,9 +55,7 @@ def main(args):
     try:
         log.debug("initializing the IOManager")
         # TODO: move this into Session
-        iomanager = IOManager(
-            report_size=(max(args.cache, 0) + 1),
-            working_path=args.working_path)
+        iomanager = IOManager(report_size=(max(args.cache, 0) + 1))
 
         log.debug("initializing Adapter %r", args.adapter)
         adapter = get_adapter(args.adapter)()
@@ -73,11 +77,13 @@ def main(args):
             args.launch_timeout,
             args.log_limit,
             args.memory,
-            args.prefs,
             relaunch,
             rr=args.rr,
             valgrind=args.valgrind,
             xvfb=args.xvfb)
+        if args.prefs:
+            target.prefs = args.prefs
+            log.info("Using prefs %r", args.prefs)
         adapter.monitor = target.monitor
 
         if args.coverage and relaunch == 1 and target.forced_close:
@@ -94,12 +100,12 @@ def main(args):
         log.debug("initializing the Reporter")
         if args.fuzzmanager:
             log.info("Results will be reported via FuzzManager")
-            reporter = FuzzManagerReporter(args.binary, tool=args.tool)
+            reporter = FuzzManagerReporter(tool=args.tool)
         elif args.s3_fuzzmanager:
             log.info("Results will be reported via FuzzManager w/ large attachments in S3")
-            reporter = S3FuzzManagerReporter(args.binary, tool=args.tool)
+            reporter = S3FuzzManagerReporter(tool=args.tool)
         else:
-            reporter = FilesystemReporter()
+            reporter = FilesystemReporter(pathjoin(getcwd(), "results"))
             log.info("Results will be stored in %r", reporter.report_path)
 
         # set 'auto_close=1' so the client error pages (code 4XX) will
@@ -126,7 +132,13 @@ def main(args):
         log.info("Ctrl+C detected.")
         return Session.EXIT_ABORT
 
-    except (TargetLaunchError, TargetLaunchTimeout):
+    except (TargetLaunchError, TargetLaunchTimeout) as exc:
+        log.error(str(exc))
+        if isinstance(exc, TargetLaunchError) and exc.report:
+            path = grz_tmp("launch_failures")
+            log.error("Logs can be found here %r", path)
+            reporter = FilesystemReporter(path, major_bucket=False)
+            reporter.submit([], exc.report)
         return Session.EXIT_LAUNCH_FAILURE
 
     finally:
