@@ -2,12 +2,10 @@
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
-import sys
+from sys import exit as sysexit
 
-from .reduce import ReductionJob
 from .args import ReducerArgs
-from ..main import console_init_logging
+from .reduce import ReductionJob
 
 
-console_init_logging()
-sys.exit(ReductionJob.main(ReducerArgs().parse_args()))
+sysexit(ReductionJob.main(ReducerArgs().parse_args()))
