@@ -6,7 +6,6 @@
 Given a build and testcase, try to reproduce it using a set of strategies.
 """
 from __future__ import absolute_import
-import glob
 import hashlib
 import io
 import json
@@ -26,9 +25,10 @@ from FTB.Signatures.CrashInfo import CrashSignature
 from . import strategies as strategies_module, testcase_contents
 from .exceptions import CorruptTestcaseError, NoTestcaseError, ReducerError
 from ..common.reporter import FilesystemReporter, FuzzManagerReporter, Report
-from ..common.runner import Runner
+from ..common.runner import Runner, RunResult
 from ..common.status import ReducerStats, Status
 from ..common.storage import TestCase, TestFile
+from ..common.utils import grz_tmp
 from ..main import configure_logging
 from ..session import Session
 from ..target import load as load_target, sanitizer_opts, TargetLaunchError, \
@@ -39,7 +39,7 @@ __author__ = "Jesse Schwartzentruber"
 __credits__ = ["Tyson Smith", "Jesse Schwartzentruber", "Jason Kratzer"]
 
 
-LOG = logging.getLogger("grizzly.reduce")
+LOG = logging.getLogger(__name__)
 
 
 class LithiumInterestingProxy(object):
@@ -244,11 +244,11 @@ class TestcaseUpdateProxy(object):
 
 class ReductionJob(object):
     LOGGERS_TO_WATCH = ("ffpuppet", "grizzly", "lithium", "sapphire")
-    DEFAULT_STRATEGIES = ("line", "jsbeautify", "collapsebraces", "jschar")
+    DEFAULT_STRATEGIES = ("line", "cssbeautify", "jsbeautify", "collapsebraces", "jschar")
     __slots__ = [
         '_any_crash', '_best_testcase', '_cache_iter_harness_created', '_env_mod',
         '_fixed_timeout', '_force_no_harness', '_idle_threshold', '_idle_timeout', '_ignore',
-        '_input_fname', '_interesting_prefix', '_iter_timeout', '_landing_page', '_log_handler',
+        '_input_fname', '_interesting_report', '_iter_timeout', '_landing_page', '_log_handler',
         '_min_crashes', '_no_harness', '_orig_sig', '_original_relaunch', '_other_crashes',
         '_reduce_file', '_repeat', '_reporter', '_result_cache', '_result_code', '_server', '_server_map',
         '_signature', '_skip', '_skip_analysis', '_skipped', '_status', '_target', '_tcroot', '_testcase',
@@ -256,8 +256,7 @@ class ReductionJob(object):
     ]
 
     def __init__(self, ignore, target, iter_timeout, no_harness, any_crash, skip, min_crashes,
-                 repeat, idle_threshold, idle_timeout, status, working_path=None,
-                 testcase_cache=True, skip_analysis=False):
+                 repeat, idle_threshold, idle_timeout, testcase_cache=True, skip_analysis=False):
         """Use lithium to reduce a testcase.
 
         Args:
@@ -273,7 +272,7 @@ class ReductionJob(object):
         self._idle_timeout = idle_timeout
         self._ignore = ignore  # things to ignore
         self._input_fname = None
-        self._interesting_prefix = None
+        self._interesting_report = None
         self._iter_timeout = iter_timeout
         self._landing_page = None  # the file to point the target at
         self._min_crashes = min_crashes
@@ -292,13 +291,13 @@ class ReductionJob(object):
         self._skip = skip
         self._skip_analysis = skip_analysis
         self._skipped = None
-        self._status = status  # ReduceStatus to track progress
+        self._status = Status.start()
         self._target = target  # a Puppet to run with
         self._testcase = None
         # testcase cache remembers if we have seen this reduce_file before and if so return the same
         # interesting result
         self._use_result_cache = testcase_cache
-        self._tmpdir = tempfile.mkdtemp(prefix="grzreduce", dir=working_path)
+        self._tmpdir = tempfile.mkdtemp(prefix="grzreduce", dir=grz_tmp("reduce"))
         self._tcroot = os.path.join(self._tmpdir, "tc")
         self._log_handler = self._start_log_capture()
         if not self._skip_analysis:
@@ -448,16 +447,6 @@ class ReductionJob(object):
                 result = self._result_cache[cache_key]['result']
                 if result:
                     LOG.info("Interesting (cached)")
-                    cached_prefix = self._result_cache[cache_key]['prefix']
-                    for filename in glob.glob(r"%s_*" % cached_prefix):
-                        suffix = os.path.basename(filename).split("_", 1)
-                        if os.path.isfile(filename):
-                            shutil.copy(filename, "%s_%s" % (temp_prefix, suffix[1]))
-                        elif os.path.isdir(filename):
-                            shutil.copytree(filename, "%s_%s" % (temp_prefix, suffix[1]))
-                        else:
-                            raise RuntimeError("Cannot copy non-file/non-directory: %s"
-                                               % (filename,))
                 else:
                     LOG.info("Uninteresting (cached)")
                 return result
@@ -487,16 +476,17 @@ class ReductionJob(object):
             self._status.report()
             self._status.iteration += 1
             run_prefix = "%s(%d)" % (temp_prefix, try_num)
-            if self._run(testcase, run_prefix):
+            interesting_report = self._run(testcase, run_prefix)
+            if interesting_report:
                 # track the maximum duration of the successful reduction attempts
                 if testcase.duration > max_duration:
                     max_duration = testcase.duration
                 n_crashes += 1
                 if n_crashes >= self._min_crashes:
-                    self.on_interesting_crash(run_prefix)
+                    self.on_interesting_crash(interesting_report)
                     if self._use_result_cache:
                         self._result_cache[cache_key] = {
-                            'result': True,
+                            'result': interesting_report,
                             'prefix': run_prefix
                         }
                     self._best_testcase = testcase
@@ -509,7 +499,7 @@ class ReductionJob(object):
             # No need to save the temp_prefix on uninteresting testcases
             # But let's do it anyway to stay consistent
             self._result_cache[cache_key] = {
-                'result': False,
+                'result': None,
                 'prefix': run_prefix
             }
         return False
@@ -597,9 +587,9 @@ class ReductionJob(object):
             temp_prefix (str): A unique prefix for any files written during this iteration.
 
         Returns:
-            bool: True if reduced testcase is still interesting.
+            Report: Report from reduced testcase if still interesting else None.
         """
-        result = False
+        interesting = None
 
         # if target is closed and server is alive, we should restart it or else the first request
         #   against /first_test will 404
@@ -656,12 +646,13 @@ class ReductionJob(object):
             self._server_map.set_redirect("grz_next_test", str(self.landing_page), required=True)
 
         # run test case
-        runner.run(self._ignore, self._server_map, testcase, wait_for_callback=self._no_harness)
+        result = runner.run(self._ignore, self._server_map, testcase, wait_for_callback=self._no_harness)
+        testcase.duration = result.duration
 
         # handle failure if detected
-        if runner.result == Runner.FAILED:
+        if result.status == RunResult.FAILED:
             self._target.close()
-            testcase.purge_optional(runner.served)
+            testcase.purge_optional(result.served)
 
             # save logs
             result_logs = temp_prefix + "_logs"
@@ -669,25 +660,23 @@ class ReductionJob(object):
                 os.mkdir(result_logs)
             self._target.save_logs(result_logs)
 
-            # create a CrashInfo
-            crash = FuzzManagerReporter.create_crash_info(
-                Report.from_path(result_logs),
-                self._target.binary)
+            # create report
+            report = Report(result_logs, self._target.binary)
 
-            short_sig = crash.createShortSignature()
+            short_sig = report.crash_info.createShortSignature()
             if short_sig == "No crash detected":
                 # XXX: need to change this to support reducing timeouts?
                 LOG.info("Uninteresting: no crash detected")
-            elif self._orig_sig is None or self._orig_sig.matches(crash):
-                result = True
+            elif self._orig_sig is None or self._orig_sig.matches(report.crash_info):
+                interesting = report
                 LOG.info("Interesting: %s", short_sig)
                 if self._orig_sig is None and not self._any_crash:
-                    self._orig_sig = Report.crash_signature(crash)
+                    self._orig_sig = report.crash_signature
             else:
                 LOG.info("Uninteresting: different signature: %s", short_sig)
-                self.on_other_crash_found(testcase, temp_prefix)
+                self.on_other_crash_found(testcase, report)
 
-        elif runner.result == Runner.IGNORED:
+        elif result.status == RunResult.IGNORED:
             LOG.info("Uninteresting: ignored")
             self._target.close()
 
@@ -697,7 +686,7 @@ class ReductionJob(object):
         # trigger relaunch by closing the browser if needed
         self._target.check_relaunch()
 
-        return result
+        return interesting
 
     def _stop_log_capture(self):
         """Stop handling reduce logs.
@@ -954,43 +943,41 @@ class ReductionJob(object):
                 self._tmpdir = None
         if self._target is not None:
             self._target.cleanup()
+        self._status.cleanup()
 
-    def _report_result(self, testcase, temp_prefix, quality_value, force=False):
+    def _report_result(self, testcase, report, quality_value, force=False):
         self._reporter.quality = quality_value
         self._reporter.force_report = force
-        self._reporter.submit([testcase], log_path=temp_prefix + "_logs")
+        self._reporter.submit([testcase], report)
 
-    def on_interesting_crash(self, temp_prefix):
+    def on_interesting_crash(self, report):
         # called for any interesting crash
-        self._interesting_prefix = temp_prefix
+        self._interesting_report = report
 
     def on_result(self, result_code):
         pass
 
-    def on_other_crash_found(self, testcase, temp_prefix):
+    def on_other_crash_found(self, testcase, report):
         """
         If we hit an alternate crash, store the testcase in a tmp folder.
         If the same crash is encountered again, only keep the newest one.
         """
-        crash_info = FuzzManagerReporter.create_crash_info(
-            Report.from_path(temp_prefix + "_logs"),
-            self._target.binary)
-        crash_hash = Report.crash_hash(crash_info)
-        if crash_hash in self._other_crashes:
-            LOG.info("Found alternate crash (newer): %s", crash_info.createShortSignature())
+        short_sig = report.crash_info.createShortSignature()
+        if report.crash_hash in self._other_crashes:
+            LOG.info("Found alternate crash (newer): %s", short_sig)
             # already counted when initially found
             self._status.ignored += 1
         else:
-            LOG.info("Found alternate crash: %s", crash_info.createShortSignature())
-            self._status.results += 1
-        self._other_crashes[crash_hash] = {"tc": testcase, "prefix": temp_prefix}
+            LOG.info("Found alternate crash: %s", short_sig)
+            self._status.count_result(short_sig)
+        self._other_crashes[report.crash_hash] = {"tc": testcase, "report": report}
 
     def _report_other_crashes(self):
         """
         After reduce is finished, report any alternate results (if they don't match the collector cache).
         """
         for entry in self._other_crashes.values():
-            self._report_result(entry["tc"], entry["prefix"], FuzzManagerReporter.QUAL_UNREDUCED)
+            self._report_result(entry["tc"], entry["report"], FuzzManagerReporter.QUAL_UNREDUCED)
 
     def run(self, strategies=None):
         """Run reduction.
@@ -1088,7 +1075,7 @@ class ReductionJob(object):
                 raise ReducerError("Reducer succeeded but nothing was reduced!")
 
             self._report_result(self._best_testcase,
-                                self._interesting_prefix,
+                                self._interesting_report,
                                 FuzzManagerReporter.QUAL_REDUCED_RESULT,
                                 force=True)
 
@@ -1113,7 +1100,7 @@ class ReductionJob(object):
         self._reporter = reporter
 
     @classmethod
-    def from_args(cls, args, target, status):
+    def from_args(cls, args, target):
         job = cls(
             args.ignore,
             target,
@@ -1125,8 +1112,6 @@ class ReductionJob(object):
             args.repeat,
             args.idle_threshold,
             args.idle_timeout,
-            status,
-            args.working_path,
             not args.no_cache,
             args.no_analysis)
 
@@ -1147,9 +1132,9 @@ class ReductionJob(object):
         LOG.debug("initializing the Reporter")
         if args.fuzzmanager:
             LOG.info("Reporting issues via FuzzManager")
-            job.set_reporter(FuzzManagerReporter(args.binary, tool=args.tool))
+            job.set_reporter(FuzzManagerReporter(tool=args.tool))
         else:
-            reporter = FilesystemReporter()
+            reporter = FilesystemReporter(os.path.join(os.getcwd(), "results"))
             job.set_reporter(reporter)
             LOG.info("Results will be stored in %r", reporter.report_path)
 
@@ -1178,7 +1163,6 @@ class ReductionJob(object):
         target = None
         job = None
 
-        status = Status.start()
         job_cancelled = False
         try:
             LOG.debug("initializing the Target")
@@ -1189,12 +1173,10 @@ class ReductionJob(object):
                 args.launch_timeout,
                 args.log_limit,
                 args.memory,
-                None,  # prefs
                 args.relaunch,
                 valgrind=args.valgrind,
                 xvfb=args.xvfb)
-
-            job = cls.from_args(args, target, status)
+            job = cls.from_args(args, target)
 
             result = job.run(strategies=args.strategies)
 
@@ -1240,8 +1222,6 @@ class ReductionJob(object):
                                                     FuzzManagerReporter.QUAL_REDUCER_ERROR}
             if job is not None:
                 job.close(keep_temp=job_cancelled)
-            # job handles calling cleanup if it was created
-            if job is None and target is not None:
+            elif target is not None:
+                # job handles calling cleanup if it was created
                 target.cleanup()
-            # call cleanup if we are unlikely to be using status again
-            status.cleanup()
