@@ -9,8 +9,7 @@ from shutil import rmtree
 from tempfile import mkdtemp
 from time import sleep, time
 
-from .common import Report, Runner, Status, TestFile
-from .target import TargetLaunchError
+from .common import grz_tmp, Report, Runner, RunResult, Status, TestFile
 
 
 __all__ = ("SessionError", "LogOutputLimiter", "Session")
@@ -18,7 +17,7 @@ __author__ = "Tyson Smith"
 __credits__ = ["Tyson Smith", "Jesse Schwartzentruber"]
 
 
-log = getLogger("grizzly")  # pylint: disable=invalid-name
+log = getLogger(__name__)  # pylint: disable=invalid-name
 
 
 class SessionError(Exception):
@@ -107,17 +106,17 @@ class Session(object):
 
     def report_result(self):
         # create working directory for target logs
-        result_logs = mkdtemp(prefix="grz_logs_", dir=self.iomanager.working_path)
+        result_logs = mkdtemp(prefix="logs_", dir=grz_tmp("logs"))
         self.target.save_logs(result_logs)
-        report = Report.from_path(result_logs)
-        crash_info = report.crash_info(self.target.binary)
-        short_sig = crash_info.createShortSignature()
+        report = Report(result_logs, self.target.binary)
+        short_sig = report.crash_info.createShortSignature()
         log.info("Result: %s (%s:%s)", short_sig, report.major[:8], report.minor[:8])
         # order test cases newest to oldest
         self.iomanager.tests.reverse()
-        self.reporter.submit(self.iomanager.tests, report=report)
+        self.reporter.submit(self.iomanager.tests, report)
         if isdir(result_logs):
             rmtree(result_logs)
+        self.status.count_result(short_sig)
 
     def run(self, ignore, iteration_limit=None, display_mode=DISPLAY_NORMAL):
         log_limiter = LogOutputLimiter(verbose=display_mode == self.DISPLAY_VERBOSE)
@@ -156,14 +155,7 @@ class Session(object):
                         forced_close=self.target.forced_close,
                         timeout=self.adapter.TEST_DURATION)
                 log.info("Launching target")
-                try:
-                    runner.launch(location, max_retries=3, retry_delay=0)
-                except TargetLaunchError:
-                    # this result likely has nothing to do with Grizzly
-                    self.status.results += 1
-                    log.error("Target launch error. Check browser logs for details.")
-                    self.report_result()
-                    raise
+                runner.launch(location, max_retries=3, retry_delay=0)
             self.target.step()
 
             # create and populate a test case
@@ -172,32 +164,32 @@ class Session(object):
             self.display_status(log_limiter=log_limiter)
 
             # run test case
-            runner.run(ignore, self.iomanager.server_map, current_test, coverage=self.coverage)
+            result = runner.run(ignore, self.iomanager.server_map, current_test, coverage=self.coverage)
+            current_test.duration = result.duration
             # adapter callbacks
-            if runner.timeout:
+            if result.timeout:
                 log.debug("calling self.adapter.on_timeout()")
-                self.adapter.on_timeout(current_test, runner.served)
+                self.adapter.on_timeout(current_test, result.served)
             else:
                 log.debug("calling self.adapter.on_served()")
-                self.adapter.on_served(current_test, runner.served)
+                self.adapter.on_served(current_test, result.served)
             # update test case
-            if runner.result != runner.ERROR:
-                if not runner.served:
+            if result.status != RunResult.ERROR:
+                if not result.served:
                     # this can happen if the target crashes between serving test cases
                     log.info("Ignoring test case since nothing was served")
                     self.iomanager.tests.pop().cleanup()
                 elif self.adapter.IGNORE_UNSERVED:
                     log.debug("removing unserved files from the test case")
-                    current_test.purge_optional(runner.served)
+                    current_test.purge_optional(result.served)
             # process results
-            if runner.result == runner.FAILED:
-                self.status.results += 1
+            if result.status == RunResult.FAILED:
                 log.debug("result detected")
                 self.report_result()
-            elif runner.result == runner.IGNORED:
+            elif result.status == RunResult.IGNORED:
                 self.status.ignored += 1
                 log.info("Ignored (%d)", self.status.ignored)
-            elif runner.result == runner.ERROR:
+            elif result.status == RunResult.ERROR:
                 log.error("Test case was not served")
                 if not current_test.contains(current_test.landing_page):
                     log.warning("Test case is missing landing page")
