@@ -10,7 +10,7 @@ from pytest import mark, raises
 from ffpuppet import BrowserTerminatedError, BrowserTimeoutError, FFPuppet
 
 from .puppet_target import PuppetTarget
-from .target import Target, TargetLaunchError, TargetLaunchTimeout
+from .target import Target, TargetError, TargetLaunchError, TargetLaunchTimeout
 
 def test_puppet_target_01(mocker, tmp_path):
     """test creating a PuppetTarget"""
@@ -19,12 +19,12 @@ def test_puppet_target_01(mocker, tmp_path):
     fake_ffp.return_value.log_length.return_value = 562
     fake_file = tmp_path / "fake"
     fake_file.touch()
-    with PuppetTarget(str(fake_file), None, 300, 25, 5000, None, 25) as target:
+    with PuppetTarget(str(fake_file), None, 300, 25, 5000, 25) as target:
         assert target.closed
-        assert target._browser_logs is None
-        assert isfile(target.prefs)
+        assert not target._remove_prefs
         prefs_file = target.prefs
-        assert target._tmp_prefs
+        assert isfile(prefs_file)
+        assert target._remove_prefs
         assert target.detect_failure([], False) == Target.RESULT_NONE
         assert target.log_size() == 1124
         fake_ffp.return_value.log_length.assert_any_call("stderr")
@@ -37,7 +37,7 @@ def test_puppet_target_01(mocker, tmp_path):
     assert fake_ffp.return_value.clean_up.call_count == 1
     assert not isfile(prefs_file)
     # with extra args
-    with PuppetTarget(str(fake_file), None, 1, 1, 1, None, 1, rr=True, fake=1) as target:
+    with PuppetTarget(str(fake_file), None, 1, 1, 1, 1, rr=True, fake=1) as target:
         pass
 
 def test_puppet_target_02(mocker, tmp_path):
@@ -46,21 +46,29 @@ def test_puppet_target_02(mocker, tmp_path):
     fake_file = tmp_path / "fake"
     fake_file.touch()
     # test providing prefs.js
-    target = PuppetTarget(str(fake_file), None, 300, 25, 5000, str(fake_file), 35)
-    assert target.prefs == str(fake_file)
-    assert not target._tmp_prefs
-    target.launch("launch_target_page")
-    assert target._browser_logs is None
-    assert fake_ffp.return_value.launch.call_count == 1
-    assert fake_ffp.return_value.close.call_count == 0
-    fake_ffp.return_value.launch.side_effect = BrowserTimeoutError
-    with raises(TargetLaunchTimeout):
-        target.launch("launch_target_page")
-    assert fake_ffp.return_value.launch.call_count == 2
-    assert fake_ffp.return_value.close.call_count == 1
-    fake_ffp.return_value.launch.side_effect = BrowserTerminatedError
-    with raises(TargetLaunchError):
+    with PuppetTarget(str(fake_file), None, 300, 25, 5000, 35) as target:
+        target.prefs = str(fake_file)
+        assert not target._remove_prefs
+        # launch success
         target.launch("launch_target_page")
+        assert fake_ffp.return_value.launch.call_count == 1
+        assert fake_ffp.return_value.close.call_count == 0
+        target.close()
+        # launch timeout
+        fake_ffp.reset_mock()
+        fake_ffp.return_value.launch.side_effect = BrowserTimeoutError("timeout")
+        with raises(TargetLaunchTimeout, match="timeout"):
+            target.launch("launch_target_page")
+        assert fake_ffp.return_value.save_logs.call_count == 0
+        # launch failure
+        fake_ffp.reset_mock()
+        (tmp_path / "log_stderr.txt").write_text("fake log")
+        (tmp_path / "log_stdout.txt").write_text("fake log")
+        mocker.patch("grizzly.target.puppet_target.mkdtemp", autospec=True, return_value=str(tmp_path))
+        fake_ffp.return_value.launch.side_effect = BrowserTerminatedError("fail")
+        with raises(TargetLaunchError, match="fail"):
+            target.launch("launch_target_page")
+        assert fake_ffp.return_value.save_logs.call_count == 1
 
 def test_puppet_target_03(mocker, tmp_path):
     """test PuppetTarget.detect_failure()"""
@@ -71,7 +79,7 @@ def test_puppet_target_03(mocker, tmp_path):
     fake_ffp.RC_WORKER = FFPuppet.RC_WORKER
     fake_file = tmp_path / "fake"
     fake_file.touch()
-    target = PuppetTarget(str(fake_file), None, 300, 25, 5000, str(fake_file), 25)
+    target = PuppetTarget(str(fake_file), None, 300, 25, 5000, 25)
     # no failures
     fake_ffp.return_value.is_healthy.return_value = True
     fake_ffp.return_value.reason = None
@@ -175,7 +183,7 @@ def test_puppet_target_04(mocker, tmp_path):
     fake_time = mocker.patch("grizzly.target.puppet_target.time", autospec=True)
     fake_file = tmp_path / "fake"
     fake_file.touch()
-    target = PuppetTarget(str(fake_file), None, 300, 25, 5000, str(fake_file), 10)
+    target = PuppetTarget(str(fake_file), None, 300, 25, 5000, 10)
     fake_kill = mocker.patch("grizzly.target.puppet_target.kill", autospec=True)
     # not running
     target.rl_countdown = 1
@@ -246,49 +254,59 @@ def test_puppet_target_04(mocker, tmp_path):
 def test_puppet_target_05(mocker, tmp_path):
     """test is_idle()"""
     fake_ffp = mocker.patch("grizzly.target.puppet_target.FFPuppet", autospec=True)
+    fake_ffp.return_value.cpu_usage.return_value = [(999, 30), (998, 20), (997, 10)]
     fake_file = tmp_path / "fake"
     fake_file.touch()
-    target = PuppetTarget(str(fake_file), None, 300, 25, 5000, None, 10)
-    fake_ffp.return_value.cpu_usage.return_value = [(999, 30), (998, 20), (997, 10)]
-    assert not target.is_idle(0)
-    assert not target.is_idle(25)
-    assert target.is_idle(50)
+    with PuppetTarget(str(fake_file), None, 300, 25, 5000, 10) as target:
+        assert not target.is_idle(0)
+        assert not target.is_idle(25)
+        assert target.is_idle(50)
 
 def test_puppet_target_06(mocker, tmp_path):
     """test PuppetTarget.monitor"""
     fake_ffp = mocker.patch("grizzly.target.puppet_target.FFPuppet", autospec=True)
     fake_file = tmp_path / "fake"
     fake_file.touch()
-    target = PuppetTarget(str(fake_file), None, 300, 25, 5000, None, 25)
-    fake_ffp.return_value.is_running.return_value = False
-    fake_ffp.return_value.is_healthy.return_value = False
-    assert target.monitor is not None
-    assert not target.monitor.is_healthy()
-    assert not target.monitor.is_running()
-    fake_ffp.return_value.is_running.return_value = True
-    fake_ffp.return_value.is_healthy.return_value = True
-    assert target.monitor.is_healthy()
-    assert target.monitor.is_running()
-    fake_ffp.return_value.launches = 123
-    assert target.monitor.launches == 123
-    fake_ffp.return_value.log_length.return_value = 100
-    assert target.monitor.log_length("stdout") == 100
-    target.monitor.clone_log("somelog")
-    assert fake_ffp.return_value.clone_log.call_count == 1
+    with PuppetTarget(str(fake_file), None, 300, 25, 5000, 25) as target:
+        fake_ffp.return_value.is_running.return_value = False
+        fake_ffp.return_value.is_healthy.return_value = False
+        assert target.monitor is not None
+        assert not target.monitor.is_healthy()
+        assert not target.monitor.is_running()
+        fake_ffp.return_value.is_running.return_value = True
+        fake_ffp.return_value.is_healthy.return_value = True
+        assert target.monitor.is_healthy()
+        assert target.monitor.is_running()
+        fake_ffp.return_value.launches = 123
+        assert target.monitor.launches == 123
+        fake_ffp.return_value.log_length.return_value = 100
+        assert target.monitor.log_length("stdout") == 100
+        target.monitor.clone_log("somelog")
+        assert fake_ffp.return_value.clone_log.call_count == 1
 
 def test_puppet_target_07(mocker, tmp_path):
-    """test PuppetTarget with GRZ_BROWSER_LOGS set"""
-    browser_logs = (tmp_path / "browser_logs")
-    fake_getenv = mocker.patch("grizzly.target.puppet_target.getenv", autospec=True)
-    fake_getenv.return_value = str(browser_logs)
-    fake_ffp = mocker.patch("grizzly.target.puppet_target.FFPuppet", autospec=True)
+    """test PuppetTarget.prefs"""
+    mocker.patch("grizzly.target.puppet_target.FFPuppet", autospec=True)
     fake_file = tmp_path / "fake"
     fake_file.touch()
-    target = PuppetTarget(str(fake_file), None, 300, 25, 5000, None, 35)
-    target.prefs = str(fake_file)
-    target.launch("launch_target_page")
-    assert target._browser_logs == str(browser_logs)
-    assert browser_logs.is_dir()
-    target.cleanup()
-    assert fake_ffp.return_value.save_logs.call_count == 1
-    assert target._browser_logs is None
+    with PuppetTarget(str(fake_file), None, 300, 25, 5000, 25) as target:
+        # default temp prefs
+        assert not target._remove_prefs
+        prefs_file = target.prefs
+        assert isfile(prefs_file)
+        assert target._remove_prefs
+        # set prefs, remove temp
+        target.prefs = str(fake_file)
+        assert not isfile(prefs_file)
+        assert not target._remove_prefs
+        prefs_file = target.prefs
+        assert isfile(prefs_file)
+        # unset prefs (revert to temp), don't remove previously specified file
+        target.prefs = None
+        assert target._remove_prefs
+        assert isfile(prefs_file)
+        prefs_file = target.prefs
+        # set missing file
+        with raises(TargetError, match="Missing prefs.js file 'missing'"):
+            target.prefs = "missing"
+        assert not isfile(prefs_file)
