@@ -7,14 +7,14 @@
 from pytest import raises
 
 from sapphire import Sapphire
-from .common import Adapter
+from .common import Adapter, Report
 from .main import main
 from .session import Session
 from .target import TargetLaunchError
 
 
 class FakeArgs(object):
-    def __init__(self, working_path):
+    def __init__(self):
         self.binary = None
         self.input = None
         self.adapter = None
@@ -36,12 +36,11 @@ class FakeArgs(object):
         self.tool = None
         self.valgrind = False
         self.verbose = False
-        self.working_path = working_path
         self.xvfb = False
 
 # TODO: these could use call_count checks
 
-def test_main_01(tmp_path, mocker):
+def test_main_01(mocker):
     """test main()"""
     fake_adapter = mocker.Mock(spec=Adapter)
     fake_adapter.NAME = "fake"
@@ -53,11 +52,12 @@ def test_main_01(tmp_path, mocker):
     fake_session = mocker.patch("grizzly.main.Session", autospec=True)
     fake_session.return_value.server = mocker.Mock(spec=Sapphire)
     fake_session.EXIT_SUCCESS = Session.EXIT_SUCCESS
-    args = FakeArgs(str(tmp_path))
+    args = FakeArgs()
     args.adapter = "fake"
     args.coverage = True
     args.input = "fake"
     args.ignore = ["fake", "fake"]
+    args.prefs = "fake"
     args.rr = True
     args.valgrind = True
     args.xvfb = True
@@ -77,19 +77,19 @@ def test_main_01(tmp_path, mocker):
     args.s3_fuzzmanager = True
     assert main(args) == Session.EXIT_SUCCESS
 
-def test_main_02(tmp_path, mocker):
+def test_main_02(mocker):
     """test main()"""
     fake_adapter = mocker.Mock(spec=Adapter)
     mocker.patch("grizzly.main.get_adapter", return_value=lambda: fake_adapter)
     fake_session = mocker.patch("grizzly.main.Session", autospec=True)
     fake_session.EXIT_SUCCESS = Session.EXIT_SUCCESS
-    args = FakeArgs(str(tmp_path))
+    args = FakeArgs()
     args.adapter = "fake"
     fake_adapter.TEST_DURATION = args.timeout + 10
     with raises(RuntimeError, match=r"Test duration \([0-9]+s\) should be less than browser timeout \([0-9]+s\)"):
         main(args)
 
-def test_main_03(tmp_path, mocker):
+def test_main_03(mocker, tmp_path):
     """test main() exit codes"""
     fake_adapter = mocker.Mock(spec=Adapter)
     fake_adapter.TEST_DURATION = 10
@@ -102,10 +102,19 @@ def test_main_03(tmp_path, mocker):
     fake_session.EXIT_ABORT = Session.EXIT_ABORT
     fake_session.EXIT_LAUNCH_FAILURE = Session.EXIT_LAUNCH_FAILURE
     fake_session.return_value.server = mocker.Mock(spec=Sapphire)
-    args = FakeArgs(str(tmp_path))
+    args = FakeArgs()
     args.adapter = "fake"
     args.input = "fake"
     fake_session.return_value.run.side_effect = KeyboardInterrupt
     assert main(args) == Session.EXIT_ABORT
-    fake_session.return_value.run.side_effect = TargetLaunchError("test")
+    # test TargetLaunchError
+    fake_tmp = (tmp_path / "grz_tmp")
+    fake_tmp.mkdir()
+    mocker.patch("grizzly.main.grz_tmp", return_value=str(fake_tmp))
+    fake_logs = (tmp_path / "report")
+    report = mocker.Mock(spec=Report, prefix="fake_report", path=str(fake_logs))
+    fake_logs.mkdir()
+    fake_session.return_value.run.side_effect = TargetLaunchError("test", report)
     assert main(args) == Session.EXIT_LAUNCH_FAILURE
+    assert any(fake_tmp.glob("fake_report_logs"))
+    assert not fake_logs.is_dir()
