@@ -8,14 +8,30 @@ unit tests for grizzly.Session
 """
 from collections import deque
 
-import pytest
+from pytest import raises
 
 from sapphire import Sapphire, ServerMap, SERVED_ALL, SERVED_NONE, SERVED_REQUEST, SERVED_TIMEOUT
-from .common import Adapter, IOManager, Reporter, Status, TestCase
+from .common import Adapter, IOManager, Report, Reporter, RunResult, Status, TestCase
 from .session import LogOutputLimiter, Session, SessionError
 from .target import Target, TargetLaunchError
 
 
+class NullReporter(Reporter):
+    def __init__(self):
+        self.submit_calls = 0
+
+    def _post_submit(self):
+        pass
+
+    def _pre_submit(self, report):
+        pass
+
+    def _submit_report(self, report, test_cases):
+        assert isinstance(report, Report)
+        for test in test_cases:
+            assert isinstance(test, TestCase)
+        self.submit_calls += 1
+
 def test_session_01(tmp_path, mocker):
     """test Session with playback Adapter"""
     class PlaybackAdapter(Adapter):
@@ -25,18 +41,19 @@ def test_session_01(tmp_path, mocker):
         def generate(self, testcase, server_map):
             assert testcase.adapter_name == self.NAME
             testcase.input_fname = "file.bin"
+            testcase.add_from_data("test", testcase.landing_page)
             self.remaining -= 1
     Status.PATH = str(tmp_path)
     adapter = PlaybackAdapter()
     adapter.setup(None, None)
     fake_serv = mocker.Mock(spec=Sapphire, port=0x1337)
-    fake_target = mocker.Mock(spec=Target, prefs=None)
+    prefs = tmp_path / "prefs.js"
+    prefs.touch()
+    fake_target = mocker.Mock(spec=Target, prefs=str(prefs))
     # set target.log_size to test warning code path
     fake_target.log_size.return_value = Session.TARGET_LOG_SIZE_WARN + 1
     with IOManager() as iomgr:
-        def fake_serve_tc(tcase, **_):
-            return (SERVED_ALL, [tcase.landing_page])
-        fake_serv.serve_testcase = fake_serve_tc
+        fake_serv.serve_path = lambda *a, **kv: (SERVED_ALL, [iomgr.page_name(offset=-1)])
         with Session(adapter, iomgr, None, fake_serv, fake_target) as session:
             session.run([])
             assert session.status.iteration == 5
@@ -50,16 +67,19 @@ def test_session_02(tmp_path, mocker):
             self.enable_harness()
         def generate(self, testcase, server_map):
             assert testcase.adapter_name == self.NAME
+            testcase.add_from_data("test", testcase.landing_page)
     Status.PATH = str(tmp_path)
     adapter = FuzzAdapter()
     adapter.setup(None, None)
     fake_serv = mocker.Mock(spec=Sapphire, port=0x1337)
-    fake_serv.serve_testcase.side_effect = lambda tc, **_: (SERVED_ALL, [tc.landing_page])
-    fake_target = mocker.Mock(spec=Target, prefs=None, rl_reset=10)
+    prefs = tmp_path / "prefs.js"
+    prefs.touch()
+    fake_target = mocker.Mock(spec=Target, prefs=str(prefs), rl_reset=10)
     fake_target.log_size.return_value = 1000
     fake_target.monitor.launches = 1
     with IOManager() as iomgr:
         iomgr.harness = adapter.get_harness()
+        fake_serv.serve_path = lambda *a, **kv: (SERVED_ALL, [iomgr.page_name(offset=-1)])
         with Session(adapter, iomgr, None, fake_serv, fake_target) as session:
             session.run([], iteration_limit=10)
             assert session.status.iteration == 10
@@ -67,16 +87,24 @@ def test_session_02(tmp_path, mocker):
 
 def test_session_03(tmp_path, mocker):
     """test Session.dump_coverage()"""
+    class FuzzAdapter(Adapter):
+        NAME = "fuzz"
+        def setup(self, input_path, server_map):
+            self.enable_harness()
+        def generate(self, testcase, server_map):
+            assert testcase.adapter_name == self.NAME
+            testcase.add_from_data("test", testcase.landing_page)
     Status.PATH = str(tmp_path)
-    adapter = mocker.Mock(spec=Adapter, remaining=None)
+    adapter = FuzzAdapter()
+    adapter.setup(None, None)
     fake_serv = mocker.Mock(spec=Sapphire, port=0x1337)
-    fake_serv.serve_testcase.side_effect = lambda tc, **_: (SERVED_ALL, [tc.landing_page])
     fake_target = mocker.Mock(spec=Target, prefs=None, rl_reset=2)
     fake_target.log_size.return_value = 1000
     fake_target.monitor.launches = 1
     # target.launch() call will be skipped
     fake_target.closed = False
     with IOManager() as iomgr:
+        fake_serv.serve_path = lambda *a, **kv: (SERVED_ALL, [iomgr.page_name(offset=-1)])
         with Session(adapter, iomgr, None, fake_serv, fake_target, coverage=True) as session:
             session.run([], iteration_limit=2)
             assert session.status.iteration == 2
@@ -97,13 +125,13 @@ def test_session_04(tmp_path, mocker):
     fake_target = mocker.Mock(spec=Target, prefs=None)
     fake_target.monitor.launches = 1
     with IOManager() as iomgr:
-        fake_serv.serve_testcase.return_value = (SERVED_NONE, [])
+        fake_serv.serve_path.return_value = (SERVED_NONE, [])
         # test error on first iteration
         with Session(adapter, iomgr, None, fake_serv, fake_target) as session:
-            with pytest.raises(SessionError, match="Please check Adapter and Target"):
+            with raises(SessionError, match="Please check Adapter and Target"):
                 session.run([], iteration_limit=10)
         # test that we continue if error happens later on
-        fake_serv.serve_testcase.return_value = (SERVED_REQUEST, ["x"])
+        fake_serv.serve_path.return_value = (SERVED_REQUEST, ["x"])
         with Session(adapter, iomgr, None, fake_serv, fake_target) as session:
             session.status.iteration = 2
             session.run([], iteration_limit=3)
@@ -111,19 +139,16 @@ def test_session_04(tmp_path, mocker):
 def test_session_05(tmp_path, mocker):
     """test basic Session functions"""
     Status.PATH = str(tmp_path)
-    fake_adapter = mocker.Mock(spec=Adapter)
+    fake_adapter = mocker.Mock(spec=Adapter, remaining=None)
     fake_adapter.TEST_DURATION = 10
-    fake_adapter.remaining = None
-    fake_testcase = mocker.Mock(spec=TestCase, landing_page="page.htm")
+    fake_testcase = mocker.Mock(spec=TestCase, landing_page="page.htm", optional=[])
     fake_iomgr = mocker.Mock(spec=IOManager)
-    fake_iomgr.server_map = ServerMap()
+    fake_iomgr = mocker.Mock(spec=IOManager, harness=None, server_map=ServerMap())
     fake_iomgr.create_testcase.return_value = fake_testcase
-    fake_iomgr.harness = None
     fake_iomgr.tests = mocker.Mock(spec=deque)
-    fake_iomgr.working_path = str(tmp_path)
     fake_serv = mocker.Mock(spec=Sapphire, port=0x1337)
     # return SERVED_TIMEOUT to test IGNORE_UNSERVED code path
-    fake_serv.serve_testcase.return_value = (SERVED_TIMEOUT, [fake_testcase.landing_page])
+    fake_serv.serve_path.return_value = (SERVED_TIMEOUT, [fake_testcase.landing_page])
     fake_target = mocker.Mock(spec=Target, prefs=None)
     fake_target.monitor.launches = 1
     with Session(fake_adapter, fake_iomgr, None, fake_serv, fake_target) as session:
@@ -136,7 +161,7 @@ def test_session_05(tmp_path, mocker):
     assert fake_iomgr.create_testcase.call_count == 1
     assert fake_iomgr.tests.pop.call_count == 0
     assert fake_testcase.purge_optional.call_count == 1
-    assert fake_serv.serve_testcase.call_count == 1
+    assert fake_serv.serve_path.call_count == 1
     assert fake_target.launch.call_count == 1
     assert fake_target.detect_failure.call_count == 1
     assert fake_target.step.call_count == 1
@@ -147,8 +172,7 @@ def test_session_06(tmp_path, mocker):
     mocker.patch("grizzly.session.TestFile", autospec=True)
     fake_adapter = mocker.Mock(spec=Adapter)
     fake_adapter.NAME = "fake_adapter"
-    fake_iomgr = mocker.Mock(spec=IOManager)
-    fake_iomgr.server_map = ServerMap()
+    fake_iomgr = mocker.Mock(spec=IOManager, server_map=ServerMap())
     fake_iomgr.create_testcase.return_value = mocker.Mock(spec=TestCase)
     fake_target = mocker.Mock(spec=Target, prefs="fake")
     with Session(fake_adapter, fake_iomgr, None, None, fake_target) as session:
@@ -165,27 +189,20 @@ def test_session_07(tmp_path, mocker):
     Status.PATH = str(tmp_path)
     mocker.patch("grizzly.session.Report", autospec=True)
     fake_runner = mocker.patch("grizzly.session.Runner", autospec=True)
+    fake_runner.return_value.run.return_value = RunResult(["/fake/file"], 1, status=RunResult.FAILED)
     mocker.patch("grizzly.session.TestFile", autospec=True)
-    fake_adapter = mocker.Mock(spec=Adapter)
+    fake_adapter = mocker.Mock(spec=Adapter, remaining=None)
     fake_adapter.IGNORE_UNSERVED = True
     fake_adapter.TEST_DURATION = 10
-    fake_adapter.remaining = None
-    fake_iomgr = mocker.Mock(spec=IOManager)
+    fake_iomgr = mocker.Mock(spec=IOManager, harness=None, server_map=ServerMap(), tests=deque())
     fake_iomgr.create_testcase.return_value = mocker.Mock(spec=TestCase)
-    fake_iomgr.harness = None
-    fake_iomgr.server_map = ServerMap()
-    fake_iomgr.tests = deque()
-    fake_iomgr.working_path = str(tmp_path)
-    fake_reporter = mocker.Mock(spec=Reporter)
     fake_serv = mocker.Mock(spec=Sapphire, port=0x1337)
     fake_target = mocker.Mock(spec=Target, prefs="prefs.js")
     fake_target.monitor.launches = 1
-    with Session(fake_adapter, fake_iomgr, fake_reporter, fake_serv, fake_target) as session:
-        session.server.serve_testcase.return_value = SERVED_REQUEST
-        fake_runner.return_value.result = fake_runner.return_value.FAILED
-        fake_runner.return_value.served = ["/fake/file"]
-        fake_runner.return_value.timeout = False
+    reporter = NullReporter()
+    with Session(fake_adapter, fake_iomgr, reporter, fake_serv, fake_target) as session:
         session.run([], iteration_limit=1)
+        assert fake_runner.return_value.run.call_count == 1
         assert fake_adapter.on_served.call_count == 1
         assert fake_adapter.on_timeout.call_count == 0
         assert fake_iomgr.purge_tests.call_count == 1
@@ -193,21 +210,19 @@ def test_session_07(tmp_path, mocker):
         assert session.status.iteration == 1
         assert session.status.results == 1
         assert session.status.ignored == 0
-        assert fake_reporter.submit.call_count == 1
+        assert reporter.submit_calls == 1
 
 def test_session_08(tmp_path, mocker):
     """test Session.run() ignoring failures"""
     Status.PATH = str(tmp_path)
     fake_runner = mocker.patch("grizzly.session.Runner", autospec=True)
+    fake_runner.return_value.run.return_value = RunResult([], 0.1, status=RunResult.IGNORED)
     mocker.patch("grizzly.session.TestFile", autospec=True)
-    fake_adapter = mocker.Mock(spec=Adapter)
+    fake_adapter = mocker.Mock(spec=Adapter, remaining=None)
     fake_adapter.IGNORE_UNSERVED = True
     fake_adapter.TEST_DURATION = 10
-    fake_adapter.remaining = None
-    fake_iomgr = mocker.Mock(spec=IOManager)
+    fake_iomgr = mocker.Mock(spec=IOManager, harness=None, server_map=ServerMap())
     fake_iomgr.create_testcase.return_value = mocker.Mock(spec=TestCase)
-    fake_iomgr.harness = None
-    fake_iomgr.server_map = ServerMap()
     fake_iomgr.tests = mocker.Mock(spec=deque)
     fake_iomgr.tests.pop.return_value = mocker.Mock(spec=TestCase)
     fake_serv = mocker.Mock(spec=Sapphire, port=0x1337)
@@ -216,11 +231,8 @@ def test_session_08(tmp_path, mocker):
     # ignored results should not be reported so raise AssertionError if report_result is called
     mocker.patch.object(Session, 'report_result', side_effect=AssertionError)
     with Session(fake_adapter, fake_iomgr, None, fake_serv, fake_target) as session:
-        session.server.serve_testcase.return_value = SERVED_REQUEST
-        fake_runner.return_value.result = fake_runner.return_value.IGNORED
-        fake_runner.return_value.served = []
-        fake_runner.return_value.timeout = False
         session.run([], iteration_limit=1)
+        assert fake_runner.return_value.run.call_count == 1
         assert fake_adapter.on_served.call_count == 1
         assert fake_adapter.on_timeout.call_count == 0
         assert fake_iomgr.purge_tests.call_count == 1
@@ -235,26 +247,24 @@ def test_session_09(tmp_path, mocker):
     Status.PATH = str(tmp_path)
     mocker.patch("grizzly.session.Report", autospec=True)
     fake_runner = mocker.patch("grizzly.session.Runner", autospec=True)
-    fake_runner.return_value.launch.side_effect = TargetLaunchError
+    fake_runner.return_value.launch.side_effect = TargetLaunchError("test", mocker.Mock(spec=Report))
     mocker.patch("grizzly.session.TestFile", autospec=True)
     fake_adapter = mocker.Mock(spec=Adapter)
-    fake_iomgr = mocker.Mock(spec=IOManager)
-    fake_iomgr.harness = None
-    fake_iomgr.input_files = []
-    fake_iomgr.server_map = ServerMap()
-    fake_iomgr.tests = deque()
-    fake_iomgr.working_path = str(tmp_path)
-    fake_reporter = mocker.Mock(spec=Reporter)
+    fake_iomgr = mocker.Mock(
+        spec=IOManager,
+        harness=None,
+        input_files=[],
+        server_map=ServerMap(),
+        tests=deque())
     fake_serv = mocker.Mock(spec=Sapphire, port=0x1337)
     fake_target = mocker.Mock(spec=Target)
     fake_target.monitor.launches = 1
-    with Session(fake_adapter, fake_iomgr, fake_reporter, fake_serv, fake_target) as session:
-        with pytest.raises(TargetLaunchError):
+    with Session(fake_adapter, fake_iomgr, mocker.Mock(), fake_serv, fake_target) as session:
+        with raises(TargetLaunchError, match="test"):
             session.run([], iteration_limit=1)
         assert session.status.iteration == 1
-        assert session.status.results == 1
+        assert session.status.results == 0
         assert session.status.ignored == 0
-    assert fake_reporter.submit.call_count == 1
 
 def test_session_10(tmp_path, mocker):
     """test Session.report_result()"""
@@ -267,21 +277,16 @@ def test_session_10(tmp_path, mocker):
         log_fp.write(b"SEGV on unknown address 0x0 (pc 0x0 bp 0x0 sp 0x0 T0)\n")
         log_fp.write(b"    #0 0xbad000 in foo /file1.c:123:234\n")
         log_fp.write(b"    #1 0x1337dd in bar /file2.c:1806:19\n")
-    fake_report = mocker.patch("grizzly.session.Report", autospec=True)
     mocker.patch("grizzly.session.mkdtemp", autospec=True, return_value=str(tmpd))
     Status.PATH = str(tmp_path)
-    fake_iomgr = mocker.Mock(spec=IOManager)
-    fake_iomgr.tests = deque()
-    fake_iomgr.working_path = str(tmp_path)
-    fake_reporter = mocker.Mock(spec=Reporter)
+    fake_iomgr = mocker.Mock(spec=IOManager, tests=deque())
     fake_target = mocker.Mock(spec=Target, binary="bin")
-    with Session(None, fake_iomgr, fake_reporter, None, fake_target) as session:
+    reporter = NullReporter()
+    with Session(None, fake_iomgr, reporter, None, fake_target) as session:
         session.report_result()
     assert fake_target.save_logs.call_count == 1
     fake_target.save_logs.assert_called_with(str(tmpd))
-    assert fake_report.from_path.return_value.crash_info.call_count == 1
-    fake_report.from_path.return_value.crash_info.assert_called_with("bin")
-    assert fake_reporter.submit.call_count == 1
+    assert reporter.submit_calls == 1
     assert not tmpd.is_dir()
 
 def test_log_output_limiter_01(mocker):
